/*
 *  Power BI Visualizations
 *
 *  Copyright (c) Microsoft Corporation
 *  All rights reserved.
 *  MIT License
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the ""Software""), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED *AS IS*, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 *  THE SOFTWARE.
 */
var powerbi;
(function (powerbi) {
    var visuals;
    (function (visuals) {
        var DotPlot1442374105856;
        (function (DotPlot1442374105856) {
            var getAnimationDuration = visuals.AnimatorCommon.GetAnimationDuration;
            var CreateClassAndSelector = jsCommon.CssConstants.createClassAndSelector;
            var AxisScale = powerbi.visuals.axisScale;
            var PixelConverter = jsCommon.PixelConverter;
            var MaxXAxisHeight = 40;
            var LabelMargin = 15;
            var DefaultRadius = 5;
            var DefaultStrokeWidth = 1;
            var DefaultDataPointColor = "#00B8AA";
            var MinPrecision = 0;
            var MaxPrecision = 17;
            DotPlot1442374105856.DotPlotProperties = {
                general: {
                    formatString: {
                        objectName: "general",
                        propertyName: "formatString"
                    }
                },
                labels: {
                    show: {
                        objectName: "labels",
                        propertyName: "show"
                    },
                    fontSize: {
                        objectName: "labels",
                        propertyName: "fontSize"
                    },
                    labelPrecision: {
                        objectName: "labels",
                        propertyName: "labelPrecision"
                    },
                    labelDisplayUnits: {
                        objectName: "labels",
                        propertyName: "labelDisplayUnits"
                    },
                    labelColor: {
                        objectName: "labels",
                        propertyName: "color"
                    }
                },
                dataPoint: {
                    fill: {
                        objectName: "dataPoint",
                        propertyName: "fill"
                    }
                },
                categories: {
                    show: {
                        objectName: "categories",
                        propertyName: "show"
                    },
                    fontColor: {
                        objectName: "categories",
                        propertyName: "fontColor"
                    },
                    fontSize: {
                        objectName: "categories",
                        propertyName: "fontSize"
                    }
                }
            };
            var DotPlot = (function () {
                function DotPlot(options) {
                    this.DefaultMargin = {
                        top: 10,
                        bottom: 10,
                        right: 20,
                        left: 20
                    };
                    this.durationAnimations = 200;
                    this.scaleType = AxisScale.linear;
                    this.textProperties = {
                        fontFamily: 'wf_segoe-ui_normal',
                        fontSize: jsCommon.PixelConverter.toString(9),
                    };
                    this.dotPlotSelectors = {
                        svgPlotSelector: CreateClassAndSelector('dotplot'),
                        plotSelector: CreateClassAndSelector('dotplotSelector'),
                        plotGroupSelector: CreateClassAndSelector('dotplotGroup'),
                        axisSelector: CreateClassAndSelector('axisGraphicsContext'),
                        xAxisSelector: CreateClassAndSelector('x axis'),
                        circleSeletor: CreateClassAndSelector('circleSelector'),
                    };
                    this.DefaultDotPlotSettings = {
                        labelSettings: {
                            show: true,
                            precision: 2,
                            fontSize: visuals.dataLabelUtils.DefaultFontSizeInPt,
                            displayUnits: 0,
                            labelColor: visuals.dataLabelUtils.defaultLabelColor,
                        },
                        categorySettings: {
                            show: true,
                            fontColor: visuals.LegendData.DefaultLegendLabelFillColor
                        },
                        defaultDataPointColor: DefaultDataPointColor
                    };
                    if (options) {
                        if (options.svg) {
                            this.svg = options.svg;
                        }
                        if (options.animator) {
                            this.animator = options.animator;
                        }
                        this.radius = options.radius || DefaultRadius;
                        this.strokeWidth = options.strokeWidth || DefaultStrokeWidth;
                    }
                }
                DotPlot.getTooltipData = function (value) {
                    return [{
                            displayName: "Value",
                            value: value.toString()
                        }];
                };
                DotPlot.converter = function (dataView, scale, defaultMargin, defaultSetting, colors, viewport, radius) {
                    var values = dataView.categorical.values, dataPointsGroup = [], displayName = dataView.categorical.categories[0].source.displayName, objects = this.getObjectsFromDataView(dataView), settings, defaultColor = powerbi.DataViewObjects.getFillColor(objects, DotPlot1442374105856.DotPlotProperties.dataPoint.fill, colors.getColorByIndex(0).value);
                    var categories = dataView.categorical.categories[0].values.map(function (x, i) { return {
                        value: x,
                        selectionId: visuals.SelectionId.createWithId(dataView.categorical.categories[0].identity[i])
                    }; });
                    settings = {
                        categorySettings: this.getCategorySettings(objects, defaultSetting),
                        defaultDataPointColor: defaultColor,
                        labelSettings: this.parseSettings(objects, defaultSetting)
                    };
                    var categoryColumn = dataView.categorical.categories[0];
                    var diameter = 2 * radius + 1;
                    var dotsTotalHeight = viewport.height - radius - MaxXAxisHeight;
                    var maxDots = Math.floor((dotsTotalHeight - defaultMargin.top) / diameter) - 1;
                    var fontSizeInPx = PixelConverter.fromPoint(settings.labelSettings.fontSize);
                    var yScale = d3.scale.linear()
                        .domain([0, maxDots])
                        .range([dotsTotalHeight - defaultMargin.bottom, defaultMargin.top + defaultMargin.bottom]);
                    for (var _i = 0, values_1 = values; _i < values_1.length; _i++) {
                        var value = values_1[_i];
                        var min = _.min(value.values);
                        var max = _.max(value.values);
                        var color = powerbi.DataViewObjects.getFillColor(objects, DotPlot1442374105856.DotPlotProperties.dataPoint.fill, colors.getColorByIndex(0).value);
                        var length_2 = value.values.length;
                        var minDots = min / (max / maxDots);
                        var dotsScale = d3.scale.log().domain([min, max]).range([minDots === 0 ? 1 : minDots, maxDots]).clamp(true);
                        for (var k = 0; k < length_2; k++) {
                            var y = dotsScale(value.values[k]);
                            var dataPoints = [];
                            for (var level = 0; level < y; level++) {
                                dataPoints.push({
                                    x: scale(categories[k].value) + scale.rangeBand() / 2,
                                    y: yScale(level),
                                    tooltipInfo: DotPlot.getTooltipData(value.values[k].toFixed(settings.labelSettings.precision))
                                });
                            }
                            var categorySelectionId = visuals.SelectionIdBuilder.builder().withCategory(categoryColumn, k).createSelectionId();
                            var tooltipInfo = DotPlot.getTooltipData(value.values[k].toFixed(settings.labelSettings.precision));
                            dataPointsGroup.push({
                                selected: false,
                                value: value.values[k],
                                label: value.values[k],
                                color: color,
                                identity: categorySelectionId,
                                tooltipInfo: tooltipInfo,
                                dataPoints: dataPoints,
                                labelFontSize: fontSizeInPx,
                            });
                        }
                    }
                    return {
                        dataPoints: dataPointsGroup,
                        values: dataView.categorical.categories[0].values,
                        displayName: displayName,
                        categories: categories,
                        settings: settings
                    };
                };
                DotPlot.prototype.init = function (options) {
                    var element = options.element;
                    this.behavior = new DotplotBehavior();
                    this.interactivityService = visuals.createInteractivityService(options.host);
                    this.radius = DefaultRadius;
                    this.strokeWidth = DefaultStrokeWidth;
                    this.colors = options.style.colorPalette.dataColors;
                    this.svg = d3.select(element.get(0)).append('svg').classed(this.dotPlotSelectors.svgPlotSelector.class, true).style('position', 'absolute');
                    this.clearCatcher = visuals.appendClearCatcher(this.svg);
                    var axisGraphicsContext = this.svg.append('g').classed(this.dotPlotSelectors.axisSelector.class, true);
                    this.dotPlot = this.svg.append('g').classed(this.dotPlotSelectors.plotSelector.class, true);
                    this.xAxis = axisGraphicsContext.append("g").classed(this.dotPlotSelectors.xAxisSelector.class, true);
                };
                DotPlot.prototype.update = function (options) {
                    if (!options.dataViews || !options.dataViews[0])
                        return;
                    this.durationAnimations = getAnimationDuration(this.animator, options.suppressAnimations);
                    var dataView = this.dataView = options.dataViews[0];
                    var viewport = options.viewport;
                    if (!dataView ||
                        !dataView.categorical ||
                        !dataView.categorical.values ||
                        dataView.categorical.values.length < 1 ||
                        !dataView.categorical ||
                        !dataView.categorical.categories ||
                        !dataView.categorical.categories[0]) {
                        this.clearData();
                        return;
                    }
                    var viewportIn = {
                        height: (viewport.height - this.DefaultMargin.top),
                        width: (viewport.width - this.DefaultMargin.left)
                    };
                    this.svg.style({
                        height: PixelConverter.toString(viewport.height),
                        width: PixelConverter.toString(viewport.width)
                    });
                    var xAxisProperties = this.calculateAxes(viewportIn, this.textProperties, false);
                    var data = DotPlot.converter(dataView, xAxisProperties.scale, this.DefaultMargin, this.DefaultDotPlotSettings, this.colors, viewport, this.radius);
                    this.dotPlotDataView = data;
                    var dataPoints = data.dataPoints;
                    if (this.interactivityService)
                        this.interactivityService.applySelectionStateToData(dataPoints);
                    this.renderAxis(viewportIn.height - MaxXAxisHeight, xAxisProperties, data, this.durationAnimations);
                    this.drawDotPlot(dataPoints, data.settings);
                    var dataLabelsSettings = data.settings.labelSettings;
                    if (dataLabelsSettings.show) {
                        var layout = this.getEnhanchedDotplotLayout(dataLabelsSettings, viewportIn);
                        visuals.dataLabelUtils.drawDefaultLabelsForDataPointChart(dataPoints, this.svg, layout, viewportIn, !options.suppressAnimations, this.durationAnimations);
                    }
                    else {
                        visuals.dataLabelUtils.cleanDataLabels(this.svg);
                    }
                };
                DotPlot.prototype.enumerateObjectInstances = function (options) {
                    var enumeration = new visuals.ObjectEnumerationBuilder();
                    switch (options.objectName) {
                        case 'dataPoint':
                            this.enumerateDataPoints(enumeration, this.dataView);
                            break;
                        case 'labels':
                            this.enumerateDataLabels(enumeration, this.dataView);
                            break;
                        case 'categories':
                            this.enumerateCategories(enumeration, this.dataView);
                            break;
                    }
                    return enumeration.complete();
                };
                DotPlot.getObjectsFromDataView = function (dataView) {
                    if (!dataView ||
                        !dataView.metadata ||
                        !dataView.metadata.columns ||
                        !dataView.metadata.objects) {
                        return null;
                    }
                    return dataView.metadata.objects;
                };
                DotPlot.parseSettings = function (objects, defaultDotPlotSettings) {
                    var precision = this.getPrecision(objects, defaultDotPlotSettings);
                    return {
                        show: powerbi.DataViewObjects.getValue(objects, DotPlot1442374105856.DotPlotProperties.labels.show, defaultDotPlotSettings.labelSettings.show),
                        precision: precision,
                        fontSize: powerbi.DataViewObjects.getValue(objects, DotPlot1442374105856.DotPlotProperties.labels.fontSize, defaultDotPlotSettings.labelSettings.fontSize),
                        displayUnits: powerbi.DataViewObjects.getValue(objects, DotPlot1442374105856.DotPlotProperties.labels.labelDisplayUnits, defaultDotPlotSettings.labelSettings.displayUnits),
                        labelColor: powerbi.DataViewObjects.getFillColor(objects, DotPlot1442374105856.DotPlotProperties.labels.labelColor, defaultDotPlotSettings.labelSettings.labelColor),
                    };
                };
                DotPlot.getCategorySettings = function (objects, defaultDotPlotSettings) {
                    return {
                        show: powerbi.DataViewObject.getValue(objects, DotPlot1442374105856.DotPlotProperties.categories.show, defaultDotPlotSettings.categorySettings.show),
                        fontColor: powerbi.DataViewObjects.getFillColor(objects, DotPlot1442374105856.DotPlotProperties.categories.fontColor, defaultDotPlotSettings.categorySettings.fontColor)
                    };
                };
                DotPlot.getPrecision = function (objects, defaultDotPlotSettings) {
                    var precision = powerbi.DataViewObjects.getValue(objects, DotPlot1442374105856.DotPlotProperties.labels.labelPrecision, defaultDotPlotSettings.labelSettings.precision);
                    if (precision <= MinPrecision)
                        return MinPrecision;
                    if (precision >= MaxPrecision)
                        return MaxPrecision;
                    return precision;
                };
                DotPlot.prototype.drawDotPlot = function (data, setting) {
                    var selection = this.dotPlot.selectAll(this.dotPlotSelectors.plotGroupSelector.selector).data(data);
                    var hasSelection = this.interactivityService && this.interactivityService.hasSelection();
                    selection
                        .enter()
                        .append('g')
                        .attr({
                        stroke: "black",
                        "stroke-width": this.strokeWidth
                    }).
                        style("fill-opacity", function (item) { return visuals.ColumnUtil.getFillOpacity(item.selected, item.highlight, hasSelection, false); }).
                        classed(this.dotPlotSelectors.plotGroupSelector.class, true);
                    var circleSelection = selection.selectAll(this.dotPlotSelectors.circleSeletor.selector).data(function (d) { return d.dataPoints; });
                    circleSelection.enter().append('circle')
                        .classed(this.dotPlotSelectors.circleSeletor.class, true);
                    circleSelection.attr({
                        cx: function (point) { return point.x; },
                        cy: function (point) { return point.y; },
                        r: this.radius,
                        fill: setting.defaultDataPointColor
                    });
                    this.renderTooltip(selection);
                    circleSelection.exit().remove();
                    selection.exit().remove();
                    var interactivityService = this.interactivityService;
                    if (interactivityService) {
                        interactivityService.applySelectionStateToData(data);
                        var behaviorOptions = {
                            columns: selection,
                            clearCatcher: this.clearCatcher,
                            interactivityService: this.interactivityService,
                        };
                        interactivityService.bind(data, this.behavior, behaviorOptions);
                    }
                };
                DotPlot.prototype.getEnhanchedDotplotLayout = function (labelSettings, viewport) {
                    var fontSizeInPx = jsCommon.PixelConverter.fromPoint(labelSettings.fontSize);
                    var formatter = visuals.valueFormatter.create({
                        format: visuals.valueFormatter.getFormatString(this.dataView.categorical.categories[0].source, DotPlot1442374105856.DotPlotProperties.general.formatString),
                        precision: labelSettings.precision,
                        value: labelSettings.displayUnits
                    });
                    return {
                        labelText: function (d) {
                            return visuals.dataLabelUtils.getLabelFormattedText({
                                label: formatter.format(d.label),
                                fontSize: labelSettings.fontSize,
                                maxWidth: viewport.width,
                            });
                        },
                        labelLayout: {
                            x: function (d) { return d && d.dataPoints && d.dataPoints[d.dataPoints.length - 1] ? d.dataPoints[d.dataPoints.length - 1].x : 0; },
                            y: function (d) { return d && d.dataPoints && d.dataPoints[d.dataPoints.length - 1] ? d.dataPoints[d.dataPoints.length - 1].y - LabelMargin : 0; }
                        },
                        filter: function (d) {
                            return (d != null && d.label != null);
                        },
                        style: {
                            'fill': labelSettings.labelColor,
                            'font-size': fontSizeInPx,
                        },
                    };
                };
                DotPlot.prototype.enumerateDataLabels = function (enumeration, dataView) {
                    var objects = dataView && dataView.metadata ? dataView.metadata.objects : undefined;
                    enumeration.pushInstance({
                        objectName: "labels",
                        displayName: "Labels",
                        selector: null,
                        properties: {
                            show: powerbi.DataViewObjects.getValue(objects, DotPlot1442374105856.DotPlotProperties.labels.show, this.DefaultDotPlotSettings.labelSettings.show),
                            fontSize: powerbi.DataViewObjects.getValue(objects, DotPlot1442374105856.DotPlotProperties.labels.fontSize, this.DefaultDotPlotSettings.labelSettings.fontSize),
                            labelPrecision: powerbi.DataViewObjects.getValue(objects, DotPlot1442374105856.DotPlotProperties.labels.labelPrecision, this.DefaultDotPlotSettings.labelSettings.precision),
                            labelDisplayUnits: powerbi.DataViewObjects.getValue(objects, DotPlot1442374105856.DotPlotProperties.labels.labelDisplayUnits, this.DefaultDotPlotSettings.labelSettings.displayUnits),
                            color: powerbi.DataViewObjects.getFillColor(objects, DotPlot1442374105856.DotPlotProperties.labels.labelColor, this.DefaultDotPlotSettings.labelSettings.labelColor)
                        }
                    });
                };
                DotPlot.prototype.enumerateDataPoints = function (enumeration, dataView) {
                    var objects = dataView && dataView.metadata ? dataView.metadata.objects : undefined;
                    var dataPointColor = powerbi.DataViewObjects.getFillColor(objects, DotPlot1442374105856.DotPlotProperties.dataPoint.fill, this.DefaultDotPlotSettings.defaultDataPointColor);
                    enumeration.pushInstance({
                        objectName: "dataPoint",
                        displayName: "Data Points",
                        selector: null,
                        properties: {
                            fill: { solid: { color: dataPointColor } }
                        }
                    });
                };
                DotPlot.prototype.enumerateCategories = function (enumeration, dataView) {
                    var objects = dataView && dataView.metadata ? dataView.metadata.objects : undefined;
                    var categoriesSettings = DotPlot.getCategorySettings(objects, this.DefaultDotPlotSettings);
                    enumeration.pushInstance({
                        objectName: "categories",
                        displayName: "Categories",
                        selector: null,
                        properties: {
                            show: categoriesSettings.show,
                            fontSize: categoriesSettings.fontSize,
                            fontColor: categoriesSettings.fontColor
                        }
                    });
                };
                DotPlot.prototype.clearData = function () {
                    this.dotPlot.selectAll("*").remove();
                    this.xAxis.selectAll("*").remove();
                    visuals.dataLabelUtils.cleanDataLabels(this.svg);
                };
                DotPlot.prototype.renderTooltip = function (selection) {
                    visuals.TooltipManager.addTooltip(selection, function (tooltipEvent) {
                        return tooltipEvent.data.tooltipInfo;
                    });
                };
                DotPlot.prototype.calculateAxes = function (viewportIn, textProperties, scrollbarVisible) {
                    var category = this.dataView.categorical.categories && this.dataView.categorical.categories.length > 0
                        ? this.dataView.categorical.categories[0]
                        : {
                            source: undefined,
                            values: [visuals.valueFormatter.format(null)],
                            identity: undefined,
                        };
                    var visualOptions = {
                        viewport: viewportIn,
                        margin: this.DefaultMargin,
                        forcedXDomain: this.dataView.categorical.categories[0].values,
                        forceMerge: false,
                        showCategoryAxisLabel: false,
                        showValueAxisLabel: false,
                        categoryAxisScaleType: this.scaleType,
                        valueAxisScaleType: null,
                        valueAxisDisplayUnits: 0,
                        categoryAxisDisplayUnits: 0,
                        trimOrdinalDataOnOverflow: false,
                    };
                    var width = viewportIn.width;
                    var axes = this.calculateAxesProperties(viewportIn, visualOptions, category.source);
                    axes.willLabelsFit = visuals.AxisHelper.LabelLayoutStrategy.willLabelsFit(axes, width, powerbi.TextMeasurementService.measureSvgTextWidth, textProperties);
                    // If labels do not fit and we are not scrolling, try word breaking
                    axes.willLabelsWordBreak = (!axes.willLabelsFit && !scrollbarVisible) && visuals.AxisHelper.LabelLayoutStrategy.willLabelsWordBreak(axes, this.DefaultMargin, width, powerbi.TextMeasurementService.measureSvgTextWidth, powerbi.TextMeasurementService.estimateSvgTextHeight, powerbi.TextMeasurementService.getTailoredTextOrDefault, textProperties);
                    return axes;
                };
                DotPlot.prototype.calculateAxesProperties = function (viewportIn, options, metaDataColumn) {
                    var xAxisProperties = visuals.AxisHelper.createAxis({
                        pixelSpan: viewportIn.width,
                        dataDomain: options.forcedXDomain,
                        metaDataColumn: metaDataColumn,
                        formatString: visuals.valueFormatter.getFormatString(metaDataColumn, DotPlot1442374105856.DotPlotProperties.general.formatString),
                        outerPadding: 0,
                        isScalar: false,
                        isVertical: false,
                        forcedTickCount: options.forcedTickCount,
                        useTickIntervalForDisplayUnits: true,
                        isCategoryAxis: true,
                        getValueFn: function (index, type) { return index; },
                        scaleType: options.categoryAxisScaleType,
                        axisDisplayUnits: options.categoryAxisDisplayUnits
                    });
                    xAxisProperties.axisLabel = "New Label";
                    return xAxisProperties;
                };
                DotPlot.prototype.renderAxis = function (height, xAxisProperties, data, duration) {
                    this.xAxis.attr({
                        transform: visuals.SVGUtil.translate(0, height)
                    });
                    var xAxis = xAxisProperties.axis;
                    xAxis.orient('bottom');
                    this.xAxis
                        .transition()
                        .duration(duration)
                        .call(xAxis);
                };
                DotPlot.capabilities = {
                    dataRoles: [{
                            name: 'Category',
                            kind: powerbi.VisualDataRoleKind.Grouping,
                            displayName: 'Category'
                        },
                        {
                            name: "Values",
                            kind: powerbi.VisualDataRoleKind.Measure,
                            displayName: 'Values'
                        }],
                    dataViewMappings: [{
                            conditions: [
                                { 'Category': { max: 1 }, 'Values': { max: 1 } },
                            ],
                            categorical: {
                                categories: {
                                    for: { in: 'Category' },
                                    dataReductionAlgorithm: { top: {} }
                                },
                                values: {
                                    group: {
                                        by: "Series",
                                        select: [{ for: { in: "Values" } }],
                                        dataReductionAlgorithm: { top: {} }
                                    }
                                }
                            },
                        }],
                    objects: {
                        general: {
                            displayName: 'General',
                            properties: {
                                formatString: {
                                    type: { formatting: { formatString: true } },
                                },
                            },
                        },
                        dataPoint: {
                            displayName: 'Data colors',
                            properties: {
                                fill: {
                                    displayName: 'Fill',
                                    type: { fill: { solid: { color: true } } }
                                }
                            }
                        },
                        labels: {
                            displayName: "Data labels",
                            description: 'Display data label options',
                            properties: {
                                show: {
                                    displayName: 'Show',
                                    type: { bool: true }
                                },
                                showSeries: {
                                    displayName: 'Show',
                                    type: { bool: true }
                                },
                                color: {
                                    displayName: 'Color',
                                    description: 'Select color for data labels',
                                    type: { fill: { solid: { color: true } } }
                                },
                                labelDisplayUnits: {
                                    displayName: 'Display units',
                                    description: 'Select the units (millions, billions, etc.)',
                                    type: { formatting: { labelDisplayUnits: true } },
                                    suppressFormatPainterCopy: true
                                },
                                labelPrecision: {
                                    displayName: 'Decimal places',
                                    description: 'Select the number of decimal places to display',
                                    placeHolderText: 'Auto',
                                    type: { numeric: true },
                                    suppressFormatPainterCopy: true
                                },
                                showAll: {
                                    displayName: 'Customize series',
                                    type: { bool: true }
                                },
                                fontSize: {
                                    displayName: 'Text Size',
                                    type: { formatting: { fontSize: true } }
                                },
                            }
                        }
                    }
                };
                return DotPlot;
            }());
            DotPlot1442374105856.DotPlot = DotPlot;
            var DotplotBehavior = (function () {
                function DotplotBehavior() {
                }
                DotplotBehavior.prototype.bindEvents = function (options, selectionHandler) {
                    this.columns = options.columns;
                    this.clearCatcher = options.clearCatcher;
                    this.interactivityService = options.interactivityService;
                    this.columns.on('click', function (d, i) {
                        selectionHandler.handleSelection(d, d3.event.ctrlKey);
                    });
                    options.clearCatcher.on('click', function () {
                        selectionHandler.handleClearSelection();
                    });
                };
                DotplotBehavior.prototype.renderSelection = function (hasSelection) {
                    var hasHighlights = this.interactivityService.hasSelection();
                    this.columns.style("fill-opacity", function (d) { return visuals.ColumnUtil.getFillOpacity(d.selected, d.highlight, !d.highlight && hasSelection, !d.selected && hasHighlights); });
                };
                return DotplotBehavior;
            }());
            DotPlot1442374105856.DotplotBehavior = DotplotBehavior;
        })(DotPlot1442374105856 = visuals.DotPlot1442374105856 || (visuals.DotPlot1442374105856 = {}));
    })(visuals = powerbi.visuals || (powerbi.visuals = {}));
})(powerbi || (powerbi = {}));

var powerbi;
(function (powerbi) {
    var visuals;
    (function (visuals) {
        var plugins;
        (function (plugins) {
            plugins.DotPlot1442374105856 = {
                name: 'DotPlot1442374105856',
                class: 'DotPlot1442374105856',
                capabilities: powerbi.visuals.DotPlot1442374105856.DotPlot.capabilities,
                custom: true,
                create: function () { return new powerbi.visuals.DotPlot1442374105856.DotPlot(); }
            };
        })(plugins = visuals.plugins || (visuals.plugins = {}));
    })(visuals = powerbi.visuals || (powerbi.visuals = {}));
})(powerbi || (powerbi = {}));
